@@ -5,6 +5,7 @@ all_continuous = [
     Beta,
     BetaScaled,
     Cauchy,
+    ChiSquared,
     Exponential,
     Gamma,
     HalfCauchy,
@@ -19,6 +20,7 @@ all_continuous = [
     Student,
     TruncatedNormal,
     Uniform,
+    VonMises,
     Wald,
     Weibull,
 ]
