@@ -1,4 +1,5 @@
 # pylint: disable=too-many-lines
+# pylint: disable=too-many-instance-attributes
 """
 Continuous probability distributions.
 """
@@ -8,10 +9,9 @@ import numpy as np
 from scipy import stats
 from scipy.special import gamma as gammaf
 
-
-from .distributions import Continuous
-from ..utils.utils import garcia_approximation
 from ..utils.optimization import optimize_ml
+from ..utils.utils import garcia_approximation
+from .distributions import Continuous
 
 eps = np.finfo(float).eps
 
@@ -44,25 +44,80 @@ class Beta(Continuous):
     Variance  :math:`\dfrac{\alpha \beta}{(\alpha+\beta)^2(\alpha+\beta+1)}`
     ========  ==============================================================
 
+    Beta distribution has 3 alternative parameterizations. In terms of alpha and
+    beta, mean and sigma (standard deviation) or mean and kappa (concentration).
+
+    The link between the 3 alternatives is given by
+
+    .. math::
+
+       \alpha &= \mu \kappa \\
+       \beta  &= (1 - \mu) \kappa
+
+       \text{where } \kappa = \frac{\mu(1-\mu)}{\sigma^2} - 1
+
+
     Parameters
     ----------
     alpha : float
         alpha  > 0
     beta : float
         beta  > 0
+    mu : float
+        mean (0 < ``mu`` < 1).
+    sigma : float
+        standard deviation (``sigma`` < sqrt(``mu`` * (1 - ``mu``))).
+    kappa : float
+        concentration > 0
     """
 
-    def __init__(self, alpha=None, beta=None):
+    def __init__(self, alpha=None, beta=None, mu=None, sigma=None, kappa=None):
         super().__init__()
-        self.alpha = alpha
-        self.beta = beta
         self.name = "beta"
-        self.params = (self.alpha, self.beta)
-        self.param_names = ("alpha", "beta")
-        self.params_support = ((eps, np.inf), (eps, np.inf))
         self.dist = stats.beta
         self.support = (0, 1)
-        self._update_rv_frozen()
+        self.params_support = ((eps, np.inf), (eps, np.inf))
+        self.alpha, self.beta, self.param_names = self._parametrization(
+            alpha, beta, mu, sigma, kappa
+        )
+        if self.alpha is not None and self.beta is not None:
+            self._update(self.alpha, self.beta)
+
+    def _parametrization(self, alpha, beta, mu, sigma, kappa):
+        if mu is None and sigma is None:
+            names = ("alpha", "beta")
+
+        elif mu is not None and sigma is not None:
+            alpha, beta = self._from_mu_sigma(mu, sigma)
+            names = ("mu", "sigma")
+
+        elif mu is not None and kappa is not None:
+            alpha, beta = self._from_mu_kappa(mu, kappa)
+            names = ("mu", "kappa")
+
+        else:
+            raise ValueError(
+                "Incompatible parametrization. Either use alpha " "and beta, or mu and sigma."
+            )
+
+        return alpha, beta, names
+
+    def _from_mu_sigma(self, mu, sigma):
+        kappa = mu * (1 - mu) / sigma**2 - 1
+        alpha = mu * kappa
+        beta = (1 - mu) * kappa
+        return alpha, beta
+
+    def _from_mu_kappa(self, mu, kappa):
+        alpha = mu * kappa
+        beta = (1 - mu) * kappa
+        return alpha, beta
+
+    def _to_mu_sigma(self, alpha, beta):
+        alpha_plus_beta = alpha + beta
+        mu = alpha / alpha_plus_beta
+        sigma = (alpha * beta) ** 0.5 / alpha_plus_beta / (alpha_plus_beta + 1) ** 0.5
+        return mu, sigma
 
     def _get_frozen(self):
         frozen = None
@@ -73,13 +128,23 @@ class Beta(Continuous):
     def _update(self, alpha, beta):
         self.alpha = alpha
         self.beta = beta
+        self.mu, self.sigma = self._to_mu_sigma(self.alpha, self.beta)
+        self.kappa = self.mu * (1 - self.mu) / self.sigma**2 - 1
+
+        if self.param_names[0] == "alpha":
+            self.params_report = (self.alpha, self.beta)
+        elif self.param_names[1] == "sigma":
+            self.params_report = (self.mu, self.sigma)
+        elif self.param_names[1] == "kappa":
+            self.params_report = (self.mu, self.kappa)
+
         self.params = (self.alpha, self.beta)
         self._update_rv_frozen()
 
     def _fit_moments(self, mean, sigma):
-        kappa = mean * (1 - mean) / sigma**2 - 1
-        alpha = max(0.5, kappa * mean)
-        beta = max(0.5, kappa * (1 - mean))
+        alpha, beta = self._from_mu_sigma(mean, sigma)
+        alpha = max(0.5, alpha)
+        beta = max(0.5, beta)
         self._update(alpha, beta)
 
     def _fit_mle(self, sample, **kwargs):
@@ -245,6 +310,71 @@ class Cauchy(Continuous):
         self._update(alpha, beta)
 
 
+class ChiSquared(Continuous):
+    r"""
+    Chi squared  log-likelihood.
+
+    The pdf of this distribution is
+
+    .. math::
+
+       f(x \mid \nu) =
+                \frac{x^{(\nu-2)/2}e^{-x/2}}{2^{\nu/2}\Gamma(\nu/2)}
+
+    .. plot::
+        :context: close-figs
+
+        import arviz as az
+        from preliz import ChiSquared
+        az.style.use('arviz-white')
+        nus = [1., 3., 9.]
+        for nu in nus:
+                ax = ChiSquared(nu).plot_pdf(support=(0,20))
+                ax.set_ylim(0, 0.6)
+
+    ========  ===============================
+    Support   :math:`x \in [0, \infty)`
+    Mean      :math:`\nu`
+    Variance  :math:`2 \nu`
+    ========  ===============================
+
+    Parameters
+    ----------
+    nu : float
+        Degrees of freedom (nu > 0).
+    """
+
+    def __init__(self, nu=None):
+        super().__init__()
+        self.nu = nu
+        self.name = "chisquared"
+        self.params = (self.nu,)
+        self.param_names = ("nu",)
+        self.params_support = ((eps, np.inf),)
+        self.dist = stats.chi2
+        self.support = (0, np.inf)
+        self._update_rv_frozen()
+
+    def _get_frozen(self):
+        frozen = None
+        if any(self.params):
+            frozen = self.dist(self.nu)
+        return frozen
+
+    def _update(self, nu):
+        self.nu = nu
+        self.params = (self.nu,)
+        self._update_rv_frozen()
+
+    def _fit_moments(self, mean, sigma=None):  # pylint: disable=unused-argument
+        nu = mean
+        self._update(nu)
+
+    def _fit_mle(self, sample, **kwargs):
+        nu, _, _ = self.dist.fit(sample, **kwargs)
+        self._update(nu)
+
+
 class Exponential(Continuous):
     r"""
     Exponential Distribution
@@ -339,25 +469,62 @@ class Gamma(Continuous):
     Variance  :math:`\dfrac{\alpha}{\beta^2}`
     ========  ===============================
 
+    Gamma distribution has 2 alternative parameterizations. In terms of alpha and
+    beta or mu (mean) and sigma (standard deviation).
+
+    The link between the 2 alternatives is given by
+
+    .. math::
+
+       \alpha &= \frac{\mu^2}{\sigma^2} \\
+       \beta  &= \frac{\mu}{\sigma^2}
+
     Parameters
     ----------
     alpha : float
         Shape parameter (alpha > 0).
     beta : float
         Rate parameter (beta > 0).
+    mu : float
+        Mean (mu > 0).
+    sigma : float
+        Standard deviation (sigma > 0)
     """
 
-    def __init__(self, alpha=None, beta=None):
+    def __init__(self, alpha=None, beta=None, mu=None, sigma=None):
         super().__init__()
-        self.alpha = alpha
-        self.beta = beta
         self.name = "gamma"
-        self.params = (self.alpha, self.beta)
-        self.param_names = ("alpha", "beta")
-        self.params_support = ((eps, np.inf), (eps, np.inf))
         self.dist = stats.gamma
         self.support = (0, np.inf)
-        self._update_rv_frozen()
+        self.params_support = ((eps, np.inf), (eps, np.inf))
+        self.alpha, self.beta, self.param_names = self._parametrization(alpha, beta, mu, sigma)
+        if self.alpha is not None and self.beta is not None:
+            self._update(self.alpha, self.beta)
+
+    def _parametrization(self, alpha, beta, mu, sigma):
+        if mu is None and sigma is None:
+            names = ("alpha", "beta")
+
+        elif mu is not None and sigma is not None:
+            alpha, beta = self._from_mu_sigma(mu, sigma)
+            names = ("mu", "sigma")
+
+        else:
+            raise ValueError(
+                "Incompatible parametrization. Either use alpha and beta or mu and sigma."
+            )
+
+        return alpha, beta, names
+
+    def _from_mu_sigma(self, mu, sigma):
+        alpha = mu**2 / sigma**2
+        beta = mu / sigma**2
+        return alpha, beta
+
+    def _to_mu_sigma(self, alpha, beta):
+        mu = alpha / beta
+        sigma = alpha**0.5 / beta
+        return mu, sigma
 
     def _get_frozen(self):
         frozen = None
@@ -368,12 +535,18 @@ class Gamma(Continuous):
     def _update(self, alpha, beta):
         self.alpha = alpha
         self.beta = beta
+        self.mu, self.sigma = self._to_mu_sigma(self.alpha, self.beta)
+
+        if self.param_names[0] == "alpha":
+            self.params_report = (self.alpha, self.beta)
+        elif self.param_names[1] == "sigma":
+            self.params_report = (self.mu, self.sigma)
+
         self.params = (self.alpha, self.beta)
         self._update_rv_frozen()
 
     def _fit_moments(self, mean, sigma):
-        alpha = mean**2 / sigma**2
-        beta = mean / sigma**2
+        alpha, beta = self._from_mu_sigma(mean, sigma)
         self._update(alpha, beta)
 
     def _fit_mle(self, sample, **kwargs):
@@ -691,25 +864,62 @@ class InverseGamma(Continuous):
     Variance  :math:`\dfrac{\beta^2}{(\alpha-1)^2(\alpha - 2)}` for :math:`\alpha > 2`
     ========  ===============================
 
+    Inverse gamma distribution has 2 alternative parameterizations. In terms of alpha and
+    beta or mu (mean) and sigma (standard deviation).
+
+    The link between the 2 alternatives is given by
+
+    .. math::
+
+       \alpha &= \frac{\mu^2}{\sigma^2} + 2 \\
+       \beta  &= \frac{\mu^3}{\sigma^2} + \mu
+
     Parameters
     ----------
     alpha : float
         Shape parameter (alpha > 0).
     beta : float
         Scale parameter (beta > 0).
+    mu : float
+        Mean (mu > 0).
+    sigma : float
+        Standard deviation (sigma > 0)
     """
 
-    def __init__(self, alpha=None, beta=None):
+    def __init__(self, alpha=None, beta=None, mu=None, sigma=None):
         super().__init__()
-        self.alpha = alpha
-        self.beta = beta
         self.name = "inversegamma"
-        self.params = (self.alpha, self.beta)
-        self.param_names = ("alpha", "beta")
-        self.params_support = ((eps, np.inf), (eps, np.inf))
         self.dist = stats.invgamma
         self.support = (0, np.inf)
-        self._update_rv_frozen()
+        self.params_support = ((eps, np.inf), (eps, np.inf))
+        self.alpha, self.beta, self.param_names = self._parametrization(alpha, beta, mu, sigma)
+        if self.alpha is not None and self.beta is not None:
+            self._update(self.alpha, self.beta)
+
+    def _parametrization(self, alpha, beta, mu, sigma):
+        if mu is None and sigma is None:
+            names = ("alpha", "beta")
+
+        elif mu is not None and sigma is not None:
+            alpha, beta = self._from_mu_sigma(mu, sigma)
+            names = ("mu", "sigma")
+
+        else:
+            raise ValueError(
+                "Incompatible parametrization. Either use alpha and beta or mu and sigma."
+            )
+
+        return alpha, beta, names
+
+    def _from_mu_sigma(self, mu, sigma):
+        alpha = mu**2 / sigma**2 + 2
+        beta = mu**3 / sigma**2 + mu
+        return alpha, beta
+
+    def _to_mu_sigma(self, alpha, beta):
+        mu = beta / (alpha - 1)
+        sigma = beta / ((alpha - 1) * (alpha - 2) ** 0.5)
+        return mu, sigma
 
     def _get_frozen(self):
         frozen = None
@@ -720,12 +930,18 @@ class InverseGamma(Continuous):
     def _update(self, alpha, beta):
         self.alpha = alpha
         self.beta = beta
+        self.mu, self.sigma = self._to_mu_sigma(self.alpha, self.beta)
+
+        if self.param_names[0] == "alpha":
+            self.params_report = (self.alpha, self.beta)
+        elif self.param_names[1] == "sigma":
+            self.params_report = (self.mu, self.sigma)
+
         self.params = (self.alpha, self.beta)
         self._update_rv_frozen()
 
     def _fit_moments(self, mean, sigma):
-        alpha = (mean**2 / sigma**2) + 2
-        beta = (mean**3 / sigma**2) + mean
+        alpha, beta = self._from_mu_sigma(mean, sigma)
         self._update(alpha, beta)
 
     def _fit_mle(self, sample, **kwargs):
@@ -908,25 +1124,57 @@ class Normal(Continuous):
     Variance  :math:`\sigma^2`
     ========  ==========================================
 
+    Normal distribution has 2 alternative parameterizations. In terms of mean and
+    sigma(standard deviation), or mean and tau (precision).
+
+    The link between the 2 alternatives is given by
+
+    .. math::
+
+        \tau = \frac{1}{\sigma^2}
+
     Parameters
     ----------
     mu : float
         Mean.
     sigma : float
         Standard deviation (sigma > 0).
+    tau : float
+        Precision (tau > 0).
     """
 
-    def __init__(self, mu=None, sigma=None):
+    def __init__(self, mu=None, sigma=None, tau=None):
         super().__init__()
-        self.mu = mu
-        self.sigma = sigma
         self.name = "normal"
-        self.params = (self.mu, self.sigma)
-        self.param_names = ("mu", "sigma")
-        self.params_support = ((-np.inf, np.inf), (eps, np.inf))
         self.dist = stats.norm
         self.support = (-np.inf, np.inf)
-        self._update_rv_frozen()
+        self.params_support = ((-np.inf, np.inf), (eps, np.inf))
+        self.mu, self.sigma, self.param_names = self._parametrization(mu, sigma, tau)
+        if self.mu is not None and self.sigma is not None:
+            self._update(self.mu, self.sigma)
+
+    def _parametrization(self, mu, sigma, tau):
+        if tau is None:
+            names = ("mu", "sigma")
+
+        elif tau is not None:
+            sigma = self._from_tau(tau)
+            names = ("mu", "tau")
+
+        else:
+            raise ValueError(
+                "Incompatible parametrization. Either use mu and sigma, or mu and tau."
+            )
+
+        return mu, sigma, names
+
+    def _from_tau(self, tau):
+        sigma = 1 / tau**0.5
+        return sigma
+
+    def _to_tau(self, sigma):
+        tau = 1 / sigma**2
+        return tau
 
     def _get_frozen(self):
         frozen = None
@@ -937,7 +1185,14 @@ class Normal(Continuous):
     def _update(self, mu, sigma):
         self.mu = mu
         self.sigma = sigma
+        self.tau = self._to_tau(sigma)
+
+        if self.param_names[1] == "sigma":
+            self.params_report = (self.mu, self.sigma)
+        elif self.param_names[1] == "tau":
+            self.params_report = (self.mu, self.tau)
         self.params = (self.mu, self.sigma)
+
         self._update_rv_frozen()
 
     def _fit_moments(self, mean, sigma):
@@ -1130,6 +1385,15 @@ class Student(Continuous):
               :math:`\infty` for :math:`1 < \nu \le 2`, otherwise undefined
     ========  ========================
 
+    Student's T distribution has 2 alternative parameterizations. In terms of nu, mu and
+    sigma (standard deviation as nu increases) or nu, mu and lam (precision as nu increases).
+
+    The link between the 2 alternatives is given by
+
+    .. math::
+
+        \lambda = \frac{1}{\sigma^2}
+
     Parameters
     ----------
     nu : float
@@ -1139,20 +1403,42 @@ class Student(Continuous):
     sigma : float
         Scale parameter (sigma > 0). Converges to the standard deviation as nu
         increases.
+    lam : float
+        Scale parameter (lam > 0). Converges to the precision as nu increases.
     """
 
-    def __init__(self, nu=3, mu=None, sigma=None):
+    def __init__(self, nu=3, mu=None, sigma=None, lam=None):
         super().__init__()
-        self.nu = nu
-        self.mu = mu
-        self.sigma = sigma
         self.name = "student"
-        self.params = (self.nu, self.mu, self.sigma)
-        self.param_names = ("nu", "mu", "sigma")
-        self.params_support = ((eps, np.inf), (-np.inf, np.inf), (eps, np.inf))
         self.dist = stats.t
         self.support = (-np.inf, np.inf)
-        self._update_rv_frozen()
+        self.params_support = ((eps, np.inf), (-np.inf, np.inf), (eps, np.inf))
+        self.nu, self.mu, self.sigma, self.param_names = self._parametrization(nu, mu, sigma, lam)
+        if self.nu is not None and self.mu is not None and self.sigma is not None:
+            self._update(self.mu, self.sigma, self.nu)
+
+    def _parametrization(self, nu, mu, sigma, lam):
+        if lam is None:
+            names = ("nu", "mu", "sigma")
+
+        elif lam is not None:
+            sigma = self._from_lam(lam)
+            names = ("nu", "mu", "lam")
+
+        else:
+            raise ValueError(
+                "Incompatible parametrization. Either use nu, mu and sigma, or nu, mu and lam."
+            )
+
+        return nu, mu, sigma, names
+
+    def _from_lam(self, lam):
+        sigma = 1 / lam**0.5
+        return sigma
+
+    def _to_lam(self, sigma):
+        lam = 1 / sigma**2
+        return lam
 
     def _get_frozen(self):
         frozen = None
@@ -1165,7 +1451,14 @@ class Student(Continuous):
             self.nu = nu
         self.mu = mu
         self.sigma = sigma
+        self.lam = self._to_lam(sigma)
+
+        if self.param_names[2] == "sigma":
+            self.params_report = (self.nu, self.mu, self.sigma)
+        elif self.param_names[2] == "lam":
+            self.params_report = (self.nu, self.mu, self.lam)
         self.params = (self.nu, self.mu, self.sigma)
+
         self._update_rv_frozen()
 
     def _fit_moments(self, mean, sigma):
@@ -1345,6 +1638,78 @@ class Uniform(Continuous):
         self._update(lower, upper)
 
 
+class VonMises(Continuous):
+    r"""
+    Univariate VonMises log-likelihood.
+
+    The pdf of this distribution is
+
+    .. math::
+
+        f(x \mid \mu, \kappa) =
+            \frac{e^{\kappa\cos(x-\mu)}}{2\pi I_0(\kappa)}
+
+    where :math:`I_0` is the modified Bessel function of order 0.
+
+    .. plot::
+        :context: close-figs
+
+        import arviz as az
+        from preliz import VonMises
+        az.style.use('arviz-white')
+        mus = [0., 0., 0.,  -2.5]
+        kappas = [.01, 0.5, 4., 2.]
+        for mu, kappa in zip(mus, kappas):
+            VonMises(mu, kappa).plot_pdf(support=(-np.pi,np.pi))
+
+    ========  ==========================================
+    Support   :math:`x \in [-\pi, \pi]`
+    Mean      :math:`\mu`
+    Variance  :math:`1-\frac{I_1(\kappa)}{I_0(\kappa)}`
+    ========  ==========================================
+
+    Parameters
+    ----------
+    mu : float
+        Mean.
+    kappa : float
+        Concentration (:math:`\frac{1}{\kappa}` is analogous to :math:`\sigma^2`).
+    """
+
+    def __init__(self, mu=None, kappa=None):
+        super().__init__()
+        self.mu = mu
+        self.kappa = kappa
+        self.name = "vonmises"
+        self.params = (self.mu, self.kappa)
+        self.param_names = ("mu", "kappa")
+        self.params_support = ((-np.inf, np.inf), (eps, np.inf))
+        self.dist = stats.vonmises
+        self.support = (-np.pi, np.pi)
+        self._update_rv_frozen()
+
+    def _get_frozen(self):
+        frozen = None
+        if any(self.params):
+            frozen = self.dist(kappa=self.kappa, loc=self.mu)
+        return frozen
+
+    def _update(self, mu, kappa):
+        self.mu = mu
+        self.kappa = kappa
+        self.params = (self.mu, self.kappa)
+        self._update_rv_frozen()
+
+    def _fit_moments(self, mean, sigma):
+        mu = mean
+        kappa = 1 / sigma**2
+        self._update(mu, kappa)
+
+    def _fit_mle(self, sample, **kwargs):
+        kappa, mu, _ = self.dist.fit(sample, **kwargs)
+        self._update(mu, kappa)
+
+
 class Wald(Continuous):
     r"""
     Wald distribution.
